@@ -251,6 +251,9 @@ setup_vm (void)
 
 	if (!vbsp) {
 		vm = vm_find();
+		if (vm == NULL) {
+			panic("Can't get vm");
+		}
 	}
 	if (new_vcpu(vm, vbsp) == NULL) {
 		panic ("new_vcpu failed");
@@ -315,7 +318,8 @@ ap_main (void)
 	call_initfunc ("ap");
 	call_initfunc ("pcpu");
 
-	if (param_get_u32("shell", 0, &shell_apic_id) == VMMERR_SUCCESS) {
+	if (param_get_u32("shell", 0, &shell_apic_id) == VMMERR_SUCCESS &&
+	    !vm_current_is_vbsp()) {
 		if (get_apic_id() == shell_apic_id) {
 			int i;
 
@@ -340,8 +344,16 @@ ap_main (void)
 	sync_all_processors(); /* 13 */
 	sync_all_processors(); /* 14 */
 
-	run_vm();
+	if (current->vbsp && 
+	    param_get_u32("shell", 0, &shell_apic_id) == VMMERR_SUCCESS) {
+		if (get_apic_id() == shell_apic_id) {
+			for (;;) {
+				shell();
+			}
+		}
+	}
 
+	run_vm();
 	panic ("VM stopped.");
 }
 
