@@ -90,29 +90,39 @@ vm_get_name()
 	return current->vm->name;
 }
 
-u32
-vm_get_id(void)
+static u32
+vm_create_id(char *name)
 {
-	char name[VM_NAME_BUF_SIZE];
+	char buf[VM_NAME_BUF_SIZE];
 	u32 id = 0;
 	int i = 0;
 	vmmerr_t err;
 
-	while ((err = param_get_str("vm", i++, name, VM_NAME_BUF_SIZE))
+	if (strcmp(name, VM0_NAME) == 0) {
+		return 0;
+	}
+
+	while ((err = param_get_str("vm", i++, buf, VM_NAME_BUF_SIZE))
 	       != VMMERR_NOTEXIST) {
 		if (err != VMMERR_SUCCESS) {
 			continue;
 		}
-		if (strcmp(name, VM0_NAME) != 0) {
+		if (strcmp(buf, VM0_NAME) != 0) {
 			id++;
 		}
-		if (strcmp(name, vm_get_name()) == 0) {
+		if (strcmp(buf, name) == 0) {
 			return id;
 		}
 	}
 	return (u32)-1;
 }
 
+u32
+vm_get_id(void)
+{
+	return current->vm->id;
+}
+
 apic_id_t
 vm_vbsp_apic_id(void)
 {
@@ -154,23 +164,11 @@ vm_get_name_from_param(char *buf, size_t size)
 		return VMMERR_SUCCESS;
 	}
 
-	if ((err = param_get_u32("shell", 0, &apic_id))
-	    != VMMERR_NOTEXIST) {
-		if (err != VMMERR_SUCCESS) {
-			printf("Failed to get shell paramater. err 0x%x\n",
-			       err);
-		} else {
-			if (apic_id == get_apic_id()) {
-				return VMMERR_NOTEXIST;
-			}
-		}
-	}
-
 	while ((err = param_get_str("vm", i++, name, VM_NAME_BUF_SIZE))
 	       != VMMERR_NOTEXIST) {
 		if (err != VMMERR_SUCCESS) {
 			printf("Failed to get vm(%d) paramater. err 0x%x\n",
-			       i, err);
+			       i - 1, err);
 			continue;
 		}
 		snprintf(param_name, VM_NAME_BUF_SIZE + 4, "%s.cpu", name);
@@ -191,7 +189,7 @@ vm_get_name_from_param(char *buf, size_t size)
 	}
 
 	/*
-	 * CPUs not specified by <vmname>.cpu or shell parameter are VM0.
+	 * CPUs not specified by <vmname>.cpu parameter are VM0.
 	 */
 	snprintf(buf, size, "%s", VM0_NAME);
 	return VMMERR_SUCCESS;
@@ -217,6 +215,7 @@ vm_current_is_vbsp(void)
 	char param_name[VM_NAME_BUF_SIZE + 4];
 	u32 apic_id;
 	vmmerr_t err;
+	int i;
 
 	if (cpu_is_bsp()) {
 		/*
@@ -230,15 +229,29 @@ vm_current_is_vbsp(void)
 		return false;
 	}
 
-	snprintf(param_name, VM_NAME_BUF_SIZE + 4, "%s.cpu", name);
-	err = param_get_u32(param_name, 0, &apic_id);
-	if (err != VMMERR_SUCCESS) {
-		if (err != VMMERR_NOTEXIST) {
-			printf("Failed to get %s(0) paramater. "
-			       "err 0x%x\n", param_name, err);
-		}
+	if (strcmp(name, VM0_NAME) == 0) {
 		return false;
 	}
+
+	snprintf(param_name, VM_NAME_BUF_SIZE + 4, "%s.cpu", name);
+
+	i = 0;
+	while ((err = param_get_u32(param_name, i++, &apic_id))
+	       != VMMERR_NOTEXIST) {
+		if (err != VMMERR_SUCCESS) {
+			if (err != VMMERR_NOTEXIST) {
+				printf("Failed to get %s(%d) paramater. "
+				       "err 0x%x\n", param_name, i - 1, err);
+				continue;
+			}
+		}
+		if (apic_id != get_bsp_apic_id()) {
+			break;
+		}
+	}
+	if (err == VMMERR_NOTEXIST) {
+		panic("Can't found the self apic_id.\n");
+	}
 	if (apic_id == get_apic_id()) {
 		return true;
 	} else {
@@ -266,6 +279,8 @@ vm_new(void)
 		return NULL;
 	}
 
+	vm->id = vm_create_id(vm->name);
+
 	spinlock_init(&vm->vcpu_list_lock);
 	LIST2_HEAD_INIT(vm->vcpu_list);
 
@@ -296,7 +311,7 @@ vm_dump(void)
 		printf(" 0x%x", vcpu->apic_id);
 	}
 	printf("\n");
-	while (vm->gmm.get_mem_map(index++, &gphys, &len, &type)) {
+	while (vm->gmm.get_mem_map(index++, &gphys, &len, &type, NULL)) {
 		printf("    %016llx-%016llx 0x%x\n",
 		       gphys, gphys + len - 1, type);
 	}
