@@ -200,7 +200,7 @@ vt_ept_walk(phys_t gphys, void *ept_l4tbl, int req_level, int *result_level)
 	u64		*tbl;
 	int		offset;
 	u64		*pte;
-	int		cur_level;
+	int		level;
 	struct ept_tbl	*stbl;
 	void		*virt;
 	phys_t		phys;
@@ -208,8 +208,8 @@ vt_ept_walk(phys_t gphys, void *ept_l4tbl, int req_level, int *result_level)
 
 	tbl = ept_l4tbl;
 
-	for (cur_level = EPT_MAX_LEVEL; cur_level >= req_level + 1; cur_level--) {
-		offset = ept_offset(gphys, cur_level);
+	for (level = EPT_MAX_LEVEL; level >= req_level + 1; level--) {
+		offset = ept_offset(gphys, level);
 		pte = &tbl[offset];
 
 		if ((*pte & EPT_VAILED_MASK) == 0) {
@@ -224,7 +224,7 @@ vt_ept_walk(phys_t gphys, void *ept_l4tbl, int req_level, int *result_level)
 				}
 				*stbl->referenced_pte = 0;
 			}
-			if (cur_level == 2) {
+			if (level == 2) {
 				LIST2_ADD(vt_data->ept_tbl_l1_list, list, stbl);
 			}
 			stbl->referenced_pte = pte;
@@ -238,8 +238,7 @@ vt_ept_walk(phys_t gphys, void *ept_l4tbl, int req_level, int *result_level)
 				EPT_READ | EPT_WRITE | EPT_EXECUTE;
 		} else {
 			if (*pte & EPT_PS) {
-				*result_level = cur_level;
-				return pte;
+				break;
 			}
 			phys = *pte & PAGE_MASK;
 			virt = (void *)phys_to_virt(phys);
@@ -247,8 +246,8 @@ vt_ept_walk(phys_t gphys, void *ept_l4tbl, int req_level, int *result_level)
 		tbl = (u64 *)virt;
 	};
 
-	*result_level = cur_level;
-	return tbl + ept_offset(gphys, cur_level);
+	*result_level = level;
+	return tbl + ept_offset(gphys, level);
 }
 
 static void
@@ -566,7 +565,7 @@ vt_ept_map_all_memory(void)
 	phys_t len;
 	u32 type;
 
-	while (gmm_get_mem_map(index++, &gphys, &len, &type)) {
+	while (gmm_get_mem_map(index++, &gphys, &len, &type, NULL)) {
 		if (type != MEM_TYPE_AVAILABLE) {
 			continue;
 		}
