@@ -37,6 +37,7 @@
 #include "entry.h"
 #include "mm.h"
 #include "panic.h"
+#include <core/printf.h>
 #include "seg.h"
 #include "vt_internal.h"
 
@@ -46,6 +47,8 @@
 #define REALMODE_TR_ACR		0x8B
 #define REALMODE_TR_BASE	sym_to_phys (head)
 
+#define CR4_PASSTHROUGH CR4_OSXSAVE_BIT
+
 #define VMREAD(a,b) do { \
 	ulong tmp; \
  \
@@ -385,6 +388,7 @@ vt_write_control_reg (enum control_reg reg, ulong val)
 {
 	ulong cr0_0, cr0_1;
 	ulong cr4_0, cr4_1;
+	ulong host_val;
 
 	switch (reg) {
 	case CONTROL_REG_CR0:
@@ -422,6 +426,15 @@ vt_write_control_reg (enum control_reg reg, ulong val)
 		break;
 	case CONTROL_REG_CR4:
 		asm_vmwrite (VMCS_CR4_READ_SHADOW, val);
+
+		asm_rdcr4(&host_val);
+		host_val = (host_val & ~CR4_PASSTHROUGH) | (val & CR4_PASSTHROUGH);
+		asm_wrcr4(host_val);
+
+		asm_vmread(VMCS_HOST_CR4, &host_val);
+		host_val = (host_val & ~CR4_PASSTHROUGH) | (val & CR4_PASSTHROUGH);
+		asm_vmwrite(VMCS_HOST_CR4, host_val);
+
 		asm_rdmsr (MSR_IA32_VMX_CR4_FIXED0, &cr4_0);
 		asm_rdmsr (MSR_IA32_VMX_CR4_FIXED1, &cr4_1);
 		val &= cr4_1;
