@@ -561,6 +561,7 @@ map_all_guest_pages(struct io_domain *dom)
 	phys_t gphys;
 	phys_t len;
 	u32 type;
+	bool restrict_access;
 
 	/*
 	 * 0MB - 1MB area need to be mapped because
@@ -568,7 +569,11 @@ map_all_guest_pages(struct io_domain *dom)
 	 */
 	map_guest_pages(dom, 0, 0x100000);
 
-	while (gmm_get_mem_map(index++, &gphys, &len, &type)) {
+	while (gmm_get_mem_map(index++, &gphys, &len, &type,
+			       &restrict_access)) {
+		if (restrict_access) {
+			continue;
+		}
 		map_guest_pages(dom, gphys, len);
 	}
 }
